@@ -29,6 +29,7 @@ import sys
 import textwrap
 import types
 
+from . import configfile
 from . import crypto
 from .helper import dputhelper
 
@@ -92,18 +93,15 @@ def parse_changes(changes_file):
     return changes
 
 
-def read_configs(extra_config, debug):
+def read_configs(config_files, debug):
     """ Read configuration settings from config files.
 
-        :param extra_config: Filesystem path of config file to read.
+        :param config_files: Sequence of configuration files, open for reading.
         :param debug: If true, enable debugging output.
         :return: The resulting `ConfigParser` instance.
 
-        Read config files in this order:
-        * If specified on the command line, only read `extra_config`.
-        * Otherwise, read ‘/etc/dput.cf’ then ‘~/.dput.cf’.
-        The config parser will layer values.
-
+        The config parser will parse each file in `config_files`. Configuration
+        from later files overrides earlier files.
         """
     config = configparser.ConfigParser()
 
@@ -126,33 +124,21 @@ def read_configs(extra_config, debug):
     config.set('DEFAULT', 'progress_indicator', '0')
     config.set('DEFAULT', 'delayed', '')
 
-    if extra_config:
-        config_file_paths = (extra_config,)
-    else:
-        config_file_paths = ('/etc/dput.cf', os.path.expanduser("~/.dput.cf"))
-    config_file = None
-    for config_file_path in config_file_paths:
-        try:
-            config_file = open(config_file_path)
-        except IOError as e:
-            if debug:
-                sys.stderr.write(
-                        "%s: %s, skipping\n" % (e.strerror, config_file_path))
-            continue
+    for config_file in config_files:
         if debug:
             sys.stdout.write(
-                    "D: Parsing Configuration File %s\n" % config_file_path)
+                    "D: Parsing configuration file ‘{path}’\n".format(
+                        path=config_file.name))
         try:
             config.read_file(config_file)
         except configparser.ParsingError as e:
-            sys.stderr.write("Error parsing config file:\n%s\n" % str(e))
+            sys.stderr.write(textwrap.dedent("""\
+                    Error parsing configuration file:
+                    {0}
+                    """).format(str(e)))
             sys.exit(1)
         config_file.close()
-    if config_file is None:
-        sys.stderr.write(
-                "Error: Could not open any configfile, tried %s\n"
-                % (', '.join(config_file_paths)))
-        sys.exit(1)
+
     # only check for fqdn and incoming dir, rest have reasonable defaults
     error = 0
     for section in config.sections():
@@ -956,9 +942,15 @@ def main():
             sys.stdout.write("D: User-ID: %s\n" % os.getuid())
             sys.stdout.write("D: Login: %s\n" % login)
 
-    # Start Config File Parsing.
-    config = read_configs(options.config_file_path, options.debug)
-
+    # Open and parse all relevant configuration files.
+    try:
+        config_files = list(configfile.active_config_files(
+                options.config_file_path, options.debug))
+        config = read_configs(config_files, options.debug)
+    except configfile.ConfigurationError as exc:
+        sys.stderr.write("Error: {0}\n".format(str(exc)))
+        sys.stderr.write("Failed to read program configuration; aborting.\n")
+        sys.exit(1)
     if options.config_print:
         print_config(config, options.debug)
         sys.exit(os.EX_OK)
@@ -1278,7 +1270,7 @@ def main():
     return
 
 
-# Copyright © 2015–2021 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2022 Ben Finney <bignose@debian.org>
 # Copyright © 2008–2013 Y Giridhar Appaji Nag <appaji@debian.org>
 # Copyright © 2006–2008 Thomas Viehmann <tv@beamnet.de>
 # Copyright © 2000–2005 Christian Kurz <shorty@debian.org>
