@@ -405,53 +405,6 @@ def patch_tempfile_mkdtemp(testcase):
     tempfile.mkdtemp.return_value = testcase.tempfile_mkdtemp_file_double.path
 
 
-try:
-    FileNotFoundError
-    FileExistsError
-    PermissionError
-except NameError:
-    # Python 2 uses IOError.
-    def _ensure_ioerror_args(init_args, init_kwargs, errno_value):
-        result_kwargs = init_kwargs
-        result_errno = errno_value
-        result_strerror = os.strerror(errno_value)
-        result_filename = None
-        if len(init_args) >= 3:
-            result_errno = init_args[0]
-            result_filename = init_args[2]
-        if 'errno' in init_kwargs:
-            result_errno = init_kwargs['errno']
-            del result_kwargs['errno']
-        if 'filename' in init_kwargs:
-            result_filename = init_kwargs['filename']
-            del result_kwargs['filename']
-        if len(init_args) >= 2:
-            result_strerror = init_args[1]
-        if 'strerror' in init_kwargs:
-            result_strerror = init_kwargs['strerror']
-            del result_kwargs['strerror']
-        result_args = (result_errno, result_strerror, result_filename)
-        return (result_args, result_kwargs)
-
-    class FileNotFoundError(IOError):
-        def __init__(self, *args, **kwargs):
-            (args, kwargs) = _ensure_ioerror_args(
-                    args, kwargs, errno_value=errno.ENOENT)
-            super().__init__(*args, **kwargs)
-
-    class FileExistsError(IOError):
-        def __init__(self, *args, **kwargs):
-            (args, kwargs) = _ensure_ioerror_args(
-                    args, kwargs, errno_value=errno.EEXIST)
-            super().__init__(*args, **kwargs)
-
-    class PermissionError(IOError):
-        def __init__(self, *args, **kwargs):
-            (args, kwargs) = _ensure_ioerror_args(
-                    args, kwargs, errno_value=errno.EPERM)
-            super().__init__(*args, **kwargs)
-
-
 def make_fake_file_scenarios(path=None):
     """ Make a collection of scenarios for testing with fake files.
 
@@ -678,14 +631,11 @@ class os_stat_scenario(TestDoubleFunctionScenario):
 
     def _hook_notfound_error(self):
         raise FileNotFoundError(
-                self.double.path,
-                "No such file or directory: {path!r}".format(
-                    path=self.double.path))
+                errno.ENOENT, "No such file or directory", self.double.path)
 
     def _hook_denied_error(self):
         raise PermissionError(
-                self.double.path,
-                "Permission denied")
+                errno.EPERM, "Permission denied", self.double.path)
 
 
 class os_lstat_scenario(os_stat_scenario):
@@ -700,15 +650,12 @@ class os_unlink_scenario(TestDoubleFunctionScenario):
 
     def _hook_nonexist(self):
         error = FileNotFoundError(
-                self.double.path,
-                "No such file or directory: {path!r}".format(
-                    path=self.double.path))
+                errno.ENOENT, "No such file or directory", self.double.path)
         raise error
 
     def _hook_denied(self):
         error = PermissionError(
-                self.double.path,
-                "Permission denied")
+                errno.EPERM, "Permission denied", self.double.path)
         raise error
 
 
@@ -720,15 +667,12 @@ class os_rmdir_scenario(TestDoubleFunctionScenario):
 
     def _hook_nonexist(self):
         error = FileNotFoundError(
-                self.double.path,
-                "No such file or directory: {path!r}".format(
-                    path=self.double.path))
+                errno.ENOENT, "No such file or directory", self.double.path)
         raise error
 
     def _hook_denied(self):
         error = PermissionError(
-                self.double.path,
-                "Permission denied")
+                errno.EPERM, "Permission denied", self.double.path)
         raise error
 
 
@@ -740,15 +684,12 @@ class shutil_rmtree_scenario(TestDoubleFunctionScenario):
 
     def _hook_nonexist(self):
         error = FileNotFoundError(
-                self.double.path,
-                "No such file or directory: {path!r}".format(
-                    path=self.double.path))
+                errno.ENOENT, "No such file or directory", self.double.path)
         raise error
 
     def _hook_denied(self):
         error = PermissionError(
-                self.double.path,
-                "Permission denied")
+                errno.EPERM, "Permission denied", self.double.path)
         raise error
 
 
@@ -762,9 +703,7 @@ class builtins_open_scenario(TestDoubleFunctionScenario):
     def _hook_nonexist(self, mode, buffering):
         if mode.startswith('r'):
             error = FileNotFoundError(
-                    self.double.path,
-                    "No such file or directory: {path!r}".format(
-                        path=self.double.path))
+                    errno.ENOENT, "No such file or directory", self.double.path)
             raise error
         result = self.double.fake_file
         return result
@@ -772,9 +711,7 @@ class builtins_open_scenario(TestDoubleFunctionScenario):
     def _hook_exist_error(self, mode, buffering):
         if mode.startswith('w') or mode.startswith('a'):
             error = FileExistsError(
-                    self.double.path,
-                    "File already exists: {path!r}".format(
-                        path=self.double.path))
+                    errno.EEXIST, "File already exists", self.double.path)
             raise error
         result = self.double.fake_file
         return result
@@ -782,9 +719,7 @@ class builtins_open_scenario(TestDoubleFunctionScenario):
     def _hook_read_denied(self, mode, buffering):
         if mode.startswith('r'):
             error = PermissionError(
-                    self.double.path,
-                    "Read denied on {path!r}".format(
-                        path=self.double.path))
+                    errno.EPERM, "Read denied", self.double.path)
             raise error
         result = self.double.fake_file
         return result
@@ -792,9 +727,7 @@ class builtins_open_scenario(TestDoubleFunctionScenario):
     def _hook_write_denied(self, mode, buffering):
         if mode.startswith('w') or mode.startswith('a'):
             error = PermissionError(
-                    self.double.path,
-                    "Write denied on {path!r}".format(
-                        path=self.double.path))
+                    errno.EPERM, "Write denied", self.double.path)
             raise error
         result = self.double.fake_file
         return result
@@ -1663,7 +1596,7 @@ def patch_os_spawnv(testcase):
     testcase.addCleanup(func_patcher.stop)
 
 
-# Copyright © 2015–2021 Ben Finney <bignose@debian.org>
+# Copyright © 2015–2022 Ben Finney <bignose@debian.org>
 #
 # This is free software: you may copy, modify, and/or distribute this work
 # under the terms of the GNU General Public License as published by the
